package PuzzleGame;

import java.util.Random;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.Point;

/**
 * This class inherits from GameBoard, and provides a couple of methods
 * usefull for creating Tetris-clones.
 * <p>Copyright: Copyright (c) 2002</p>
 * @author Martin Karlsson, fist_of_heaven@hotmail.com
 * @version 1.0
 */
 public class TetrisGameBoard extends GameBoard
 {
 ////////////////////////////////////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////
    public TetrisGameBoard()
    {
       super();
    }

    public TetrisGameBoard(int row, int col, int gridsize)
    {
       super(row, col);
		 this.gridsize = gridsize;
    }
 ////////////////////////////////////////////////////////////////////////////////

 /*****************************************************************************
 * Checks if the positions in a row in the gameboard matrix is all true, beginning
 * on row [start] and moving up
 *@param start
 * The row to start on
 *@return
 * Returns the number of the complete row, or -1 if there's no complete row
 */
    public int checkForCompleteRow(int start)
    {

       boolean complete;
			 if (start < 1)
			 	start = 1;
       for (int i = start; i < numRow; i = i + 2)
       {
			 complete = true;
          for (int j = 0; j < numCol; j++)
			 {
             complete = (gameBoardMatrix[i][j] >= 0) && complete;
				 complete = (gameBoardMatrix[i-1][j] >= 0) && complete;
			 }

          if (complete)
             return i+1;
       }

       return -1;
    }

 /*****************************************************************************
 * Moves the values in the gameboard matrix [steps] steps down (row-wise), beginning
 * with row [last] and ending in row [first]. If [last] is set to below 1, every row from
 * the top of the gameboard to row [first] is moved down [steps] number of steps.
 * @param steps
 * The number of steps to move the portion of the gameboard
 * @param first
 * The first row to start moving down (counting bottom-up).
 * @param last
 * The last row to move down. 0 or lower if all rows above [first] is to be moved down.
 */
    public void moveDown(int last, int first, int steps)
    {

       if (last < 1)
          last = numRow;

       if (first > 0)
       {
          for (int step = 1; step <= steps; step++)
             for(int i = first; i < last; i++)
                for(int j = 1; j < numCol; j=j+2)
					 {
      				gameBoardMatrix[i-1][j] = gameBoardMatrix[i][j];
						gameBoardMatrix[i-1][j-1] = gameBoardMatrix[i][j-1];
					 }

          eraseRow(last-1);
       }

    }//moveDown

/**
 * Applies gravity on the gameboard, e.g. moves the portion of the
 * gameboard above one empty row, down one row. Goes through the complete
 * gameboard.
 */
    public int applyGravity()
    {
       int row = 1, num = 0;
       while ((row = eraseRow(checkForCompleteRow(row-1))) > 0)
       {
          moveDown(0, row, 1);
			 num = num + 1;
       }
			 return num;
    }
/**
	Sets the pieces and their corresponding image provided in the arrays
	@param piecearray
		The array containing the TetrisPuzzlePieces (or pieces that inherits from that class).
	@param imagearray
		The array containing the images that is to be used with the corresponding piece.
 */
  public void setPieces(TetrisPuzzlePiece piecearray[], Image imagearray[])
  {
			numPieces = piecearray.length;
			pieces = new TetrisPuzzlePiece[numPieces];
			images = new Image[imagearray.length];

			for (int i = 0; i < numPieces; i++)
			{
				pieces[i] = piecearray[i];
				pieces[i].setImage(imagearray[i]);
				images[i] = imagearray[i];
			}
  }

/**
* Gets a random piece from the class' piece array
* @return
*  A random Tetrispiece
*/
    public TetrisPuzzlePiece getRandomPiece()
    {
	    Random rndgen = new Random();
	    return pieces[rndgen.nextInt(numPieces)].getCopy();
    }
/**
* Checks if collision occurs between a TetrisPuzzlePiece and the gameboard.
* @param piece
*		The piece that should be used in the test of boundaries
* @return
* 	1 if collision with borders, 2 if collision with other
*		pieces in the matrix, 0 otherwise.
*/
	public int checkForCollision(TetrisPuzzlePiece piece)
	{
		int piecematrix[][] = piece.getMatrix();
		int col = piece.getLocation().y;
		int row = gameBoardMatrix.length - piece.getLocation().x - 1;

		//Check with boundaries
		if ( (piece.getRightmostPosition() >= gameBoardMatrix[0].length) ||
				 (piece.getBottommostPosition() >= gameBoardMatrix.length) ||
				 (piece.getLocation().x < 0) ||
				 (piece.getLocation().y < 0) )
			return 1;

		int tempmatrix[][] = piece.getMatrix();
		int boardrow = gameBoardMatrix.length - piece.getLocation().x - 1;

		//Check with other pieces
		for (int i = 0; i < tempmatrix.length; i++)
			for (int j = 0; j < tempmatrix[0].length; j++)
			{
				if ( (tempmatrix[i][j] > 0) &&
						 (gameBoardMatrix[boardrow - i][piece.getLocation().y + j] >= 0) )
					return 2;
			}

		return 0;
	}

/**
 * Draws the board with an offset. Gets tiles from the image-array
 * @param offsetx
 		The x-axis offset on the provided graphics page
 * @param offsety
 		The y-axis offset on the provided graphics page
 * @param graphicspage
 		The graphics page to draw the board on
 */
	public void drawBoard(int offsetx, int offsety, Graphics graphicspage)
	{
		offsety = offsety + gridsize*(gameBoardMatrix.length-1);
		for (int i = 0; i < gameBoardMatrix.length; i++)
			for(int j = 0; j < gameBoardMatrix[0].length; j++)
				if (gameBoardMatrix[i][j] >= 0)
					graphicspage.drawImage(images[gameBoardMatrix[i][j]],
					offsetx + gridsize*j, offsety - gridsize*i, null);
				else
					graphicspage.clearRect(offsetx+gridsize*j, offsety-gridsize*i,images[0].getWidth(null), images[0].getHeight(null));
	}

/**
 * Adds a piece to the matrix, at the piece' current position
 * @param piece
 		The piece to add to the matrix
 */

	public void updateBoardWith(TetrisPuzzlePiece piece)
	{
		int col = piece.getLocation().y;
		int row = gameBoardMatrix.length - piece.getLocation().x - 1;
		int image_nbr;

		//Determine which image in the image-array that is set to this piece
		for (image_nbr = 0; image_nbr < images.length; image_nbr++)
			if (images[image_nbr] == piece.getImage())
				break;

		for (int i = 0; i < piece.numRow; i++)
			for (int j = 0; j < piece.numCol; j++)
				if (piece.pieceMatrix[i][j] != 0)
				{
					gameBoardMatrix[row - i][col + j] = image_nbr;
				}
	}

    protected TetrisPuzzlePiece pieces[];
		protected Image images[];
		protected Image borders[];
		protected int numPieces = 0;
		private int gridsize = 0;
}
